/*
 * Copyright (C) 2015 Adrian Ulrich <adrian@blinkenlights.ch>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>. 
 */


package ch.blinkenlights.android.vanilla;

import android.content.Context;
import android.content.Intent;
import android.media.MediaPlayer;
import android.media.audiofx.AudioEffect;

import java.io.IOException;


import android.util.Log;


public class VanillaMediaPlayer extends MediaPlayer {

	private Context mContext;
	private String mDataSource;
	private int mClaimedAudioSessionId = 0;

	/**
	 * Constructs a new VanillaMediaPlayer class
	 */
	public VanillaMediaPlayer(Context context) {
		super();
		mContext = context;
		_claimAudioSession();
	}

	/**
	 * Resets the media player to an unconfigured state
	 */
	public void reset() {
		mDataSource = null;
		super.reset();
	}

	/**
	 * Releases the media player and frees any claimed AudioEffect
	 */
	public void release() {
		_releaseAudioSession();
		mDataSource = null;
		super.release();
	}

	/**
	 * Sets the data source to use
	 */
	public void setDataSource(String dataSource) throws IOException, IllegalArgumentException, SecurityException, IllegalStateException {
		mDataSource = dataSource;
		super.setDataSource(mDataSource);
		_claimAudioSession();
	}

	/**
	 * Returns the configured data source, may be null
	 */
	public String getDataSource() {
		return mDataSource;
	}

	/**
	 * Creates a new AudioEffect for our AudioSession
	 */
	private void _claimAudioSession() {
		// No active audio session -> claim one
		Log.v("VanillaMusic", "_claimAudioSession() -> "+mClaimedAudioSessionId+" -> "+this);
		if (mClaimedAudioSessionId == 0) {
			mClaimedAudioSessionId = this.getAudioSessionId();
			Intent i = new Intent(AudioEffect.ACTION_OPEN_AUDIO_EFFECT_CONTROL_SESSION);
			i.putExtra(AudioEffect.EXTRA_AUDIO_SESSION, mClaimedAudioSessionId);
			i.putExtra(AudioEffect.EXTRA_PACKAGE_NAME, mContext.getPackageName());
			mContext.sendBroadcast(i);
		}
	}

	/**
	 * Releases a previously claimed audio session id
	 */
	private void _releaseAudioSession() {
		Log.v("VanillaMusic", "_releaseAudioSession() "+mClaimedAudioSessionId+" -> "+this);

		if (mClaimedAudioSessionId != 0) {
			Intent i = new Intent(AudioEffect.ACTION_CLOSE_AUDIO_EFFECT_CONTROL_SESSION);
			i.putExtra(AudioEffect.EXTRA_AUDIO_SESSION, mClaimedAudioSessionId);
			i.putExtra(AudioEffect.EXTRA_PACKAGE_NAME, mContext.getPackageName());
			mContext.sendBroadcast(i);
			mClaimedAudioSessionId = 0;
		}
	}

}
